/*
 * $Id: Bullet.cs,v 1.1.1.1 2009/08/10 14:55:11 kenta Exp $
 *
 * Copyright 2009 Kenta Cho. Some rights reserved.
 */
using System;
using System.Runtime.InteropServices;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Abagames.Util;

namespace Abagames.GearToyGear
{
    /// <summary>
    /// Enemy's bullets.
    /// </summary>
    public class BulletPool : ActorPool<Bullet>
    {
        private Frame frame;
        private Field field;
        private Player player;
        private PillarPool pillars;
        private ParticlePool particles;
        private EllipseEdgeShape edgeShape;
        private Quaternion rollQuaternion;

        public BulletPool(int n, Frame frame, Field field, Player player, PillarPool pillars, ParticlePool particles)
            : base(n)
        {
            this.frame = frame;
            this.field = field;
            this.player = player;
            this.pillars = pillars;
            this.particles = particles;
            shape = new EllipseShape(frame, 1, 0.6f);
            edgeShape = new EllipseEdgeShape(frame, 1, 0.6f, 0.8f, 16);
            rollQuaternion = Quaternion.CreateFromYawPitchRoll(0, 0, 0.1f);
        }

        public void Add(Vector3 from, Vector3 to, float speed)
        {
            Bullet a = new Bullet();
            a.Pos = from;
            a.Vel = to - from;
            a.Vel.Normalize();
            a.Vel *= speed;
            a.Scale = 8.0f;
            a.Orientation = Quaternion.Identity;
            a.Color.X = 0.5f;
            a.Color.Y = 1;
            a.Color.Z = 1;
            a.Color.W = 0.7f;
            Add(ref a);
        }

        public void Add(Vector3 from, float speed)
        {
            Bullet a = new Bullet();
            a.Pos = from;
            a.Vel.Z = speed;
            a.Scale = 8.0f;
            a.Orientation = Quaternion.Identity;
            a.Color.X = 0.5f;
            a.Color.Y = 1;
            a.Color.Z = 1;
            a.Color.W = 0.7f;
            Add(ref a);
        }

        public override bool Update(ref Bullet a)
        {
            a.Orientation *= rollQuaternion;
            a.Pos += a.Vel * Stage.GameSpeed;
            if (pillars.CheckHit(a.Pos))
            {
                particles.Add(16, a.Pos, a.Vel, 0.5f, 10, 0.4f, 1.0f, 0.8f, 0.6f);
                return false;
            }
            if (a.Pos.Z > -10 && Vector3.Distance(a.Pos, player.Pos) < a.Scale * 0.5f)
            {
                player.Destroy();
                return false;
            }
            return (a.Pos.Z < Field.FrontDepth);
        }

        public override void Draw(ref Bullet a)
        {
            Shape.AddInstance(a.Pos, a.Scale, a.Orientation, a.Color);
        }

        public void DrawEdge()
        {
            Draw(edgeShape);
        }
    }

    [StructLayout(LayoutKind.Sequential)]
    public struct Bullet
    {
        public Vector3 Pos;
        public float Scale;
        public Quaternion Orientation;
        public Vector4 Color;
        public Vector3 Vel;
    }
}
