/*
 * $Id: Field.cs,v 1.1.1.1 2009/08/10 14:55:11 kenta Exp $
 *
 * Copyright 2009 Kenta Cho. Some rights reserved.
 */
using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Abagames.Util;
using System.Runtime.InteropServices;

namespace Abagames.GearToyGear
{
    /// <summary>
    /// Tube shaped game field.
    /// </summary>
    public class Field
    {
        public const float FrontDepth = 16, BackDepth = -512, FireBoundaryDepth = -120;
        public const int TubeCount = 16;
        private Frame frame;
        private Tube[] tubes;
        private GearShape tubeShape;

        public Field(Frame frame)
        {
            this.frame = frame;
            tubes = new Tube[TubeCount];
            tubeShape = new GearShape(frame, Tube.Radius * 1.2f, Tube.Radius, Tube.Height,
                new Color(100, 200, 255, 200), new Color(100, 150, 200, 200), false);
        }

        public void Initialize()
        {
            for (int i = 0; i < tubes.Length; i++)
                tubes[i].Initialize(i);
        }

        public void Update()
        {
            for (int i = 0; i < TubeCount; i++)
                tubes[i].Update();
        }

        public void Draw()
        {
            Shape.BeginAddInstance();
#if XBOX360
            Shape.SetToWritableVertexBuffer<Tube>(tubes, 0, TubeCount);
#else
            for (int i = 0; i < TubeCount; i++)
                tubes[i].Draw();
#endif
            tubeShape.Draw();
        }
    }

    [StructLayout(LayoutKind.Sequential)]
    public struct Tube
    {
        public const float Radius = 100;
        public const float Height = 15;
        private Vector3 pos;
        private float scale;
        private Quaternion orientation;
        private Vector4 color;
        private Quaternion roll;

        public void Initialize(int i)
        {
            pos.X = pos.Y = 0;
            pos.Z = -i * Height * 2 * 1.1f;
            scale = 1.0f;
            orientation = Quaternion.Identity;
            color.X = color.Y = color.Z = color.W = 0;
            roll = Quaternion.CreateFromYawPitchRoll(0, 0, ((i % 2) * 2 - 1) * 0.002f);
        }

        public void Update()
        {
            orientation *= roll;
            pos.Z += Stage.PlayerDepthSpeed;
            if (pos.Z > Field.FrontDepth)
                pos.Z -= Height * 2 * 1.1f * Field.TubeCount;
        }

        public void Draw()
        {
            Shape.AddInstance(pos, scale, orientation, color);
        }
    }
}
