/*
 * $Id: HomingLaser.cs,v 1.1.1.1 2009/08/10 14:55:11 kenta Exp $
 *
 * Copyright 2009 Kenta Cho. Some rights reserved.
 */
using System;
using System.Runtime.InteropServices;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Audio;
using Abagames.Util;

namespace Abagames.GearToyGear
{
    /// <summary>
    /// Enemy's horming lasers.
    /// </summary>
    public class HomingLaserPool : ActorPool<HomingLaser>
    {
        private Frame frame;
        private Field field;
        private LaserPool lasers;
        private Player player;
        private PillarPool pillars;
        private ParticlePool particles;
        private Sound sound;

        public HomingLaserPool(int n, Frame frame, Field field, LaserPool lasers, Player player, PillarPool pillars, ParticlePool particles, Sound sound)
            : base(n)
        {
            this.frame = frame;
            this.field = field;
            this.lasers = lasers;
            this.player = player;
            this.pillars = pillars;
            this.particles = particles;
            this.sound = sound;
        }

        public override void Clear()
        {
            ForEach(delegate(ref HomingLaser a)
            {
                if (a.Cue != null)
                    a.Cue.Stop(AudioStopOptions.Immediate);
            });
            base.Clear();
        }

        public void Add(Vector3 from, Vector3 to, float speed)
        {
            HomingLaser a = new HomingLaser();
            a.Pos = from;
            a.Vel = to - from;
            a.Vel.Normalize();
            a.Vel *= speed;
            Add(ref a);
        }

        public override bool Update(ref HomingLaser a)
        {
            a.Pos += a.Vel * Stage.GameSpeedSqrt;
            if (pillars.CheckHit(a.Pos))
            {
                particles.Add(24, a.Pos, a.Vel, 0.5f, 10, 0.8f, 1.0f, 0.4f, 0.7f);
                return false;
            }
            float tt = -a.Pos.Z / (a.Vel.Z * Stage.GameSpeed);
            if (tt < 1)
                tt = 1;
            float tx = a.Pos.X + a.Vel.X * tt;
            float ty = a.Pos.Y + a.Vel.Y * tt;
            a.Vel.X += (player.Pos.X + player.Vel.X * tt - tx) * (0.05f / tt);
            a.Vel.Y += (player.Pos.Y + player.Vel.Y * tt - ty) * (0.05f / tt);
            lasers.Add(a.Pos, a.Vel, a.Vel.Length() * 0.8f, 25);
            particles.Add(6, a.Pos, a.Vel, 0.1f, 15, 0.4f, 1, 0.2f, 0.8f);
            if (a.Pos.Z > -10 && Vector3.Distance(a.Pos, player.Pos) < 5.0f)
            {
                player.Destroy();
            }
            if (a.Cue == null)
            {
                a.Cue = sound.GetCue("HomingLaser");
                a.Listener = new AudioListener();
                a.Emitter = new AudioEmitter();
                a.Cue.Apply3D(a.Listener, a.Emitter);
                a.Cue.Play();
            }
            a.Listener.Position = sound.ListenerPos;
            a.Emitter.Position = a.Pos;
            a.Emitter.Velocity = a.Vel;
            a.Cue.Apply3D(a.Listener, a.Emitter);
            return (a.Pos.Z < Field.FrontDepth * 2);
        }

        public override void Remove(int i)
        {
            if (Actors[i].Cue != null)
            {
                Actors[i].Cue.Stop(AudioStopOptions.Immediate);
                Actors[i].Cue = null;
            }
            base.Remove(i);
        }
    }

    public struct HomingLaser
    {
        public Vector3 Pos;
        public Vector3 Vel;
        public Cue Cue;
        public AudioListener Listener;
        public AudioEmitter Emitter;
    }
}
