/*
 * $Id: Laser.cs,v 1.1.1.1 2009/08/10 14:55:11 kenta Exp $
 *
 * Copyright 2009 Kenta Cho. Some rights reserved.
 */
using System;
using System.Runtime.InteropServices;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Abagames.Util;

namespace Abagames.GearToyGear
{
    /// <summary>
    /// Enemy's lasers.
    /// </summary>
    public class LaserPool : ActorPool<Laser>
    {
        private Frame frame;
        private Field field;
        private Shape edgeShape;

        public LaserPool(int n, Frame frame, Field field)
            : base(n)
        {
            this.frame = frame;
            this.field = field;
            shape = new CubeShape(frame, 0.33f, 0.33f, 1);
            edgeShape = new CubeEdgeShape(frame, 0.33f, 0.33f, 1);
        }

        public void Add(Vector3 p, Vector3 v, float length)
        {
            Laser a = new Laser();
            a.Pos = p;
            a.Scale = length;
            a.Orientation.X = -v.Y;
            a.Orientation.Y = v.X;
            a.Orientation.Z = 0;
            a.Orientation.W = v.Z + v.Length();
            a.Orientation.Normalize();
            a.Color.X = 0.25f;
            a.Color.Y = 1;
            a.Color.Z = 0.5f;
            a.Color.W = 0.75f;
            a.Ticks = 1;
            Add(ref a);
        }

        public void Add(Vector3 p, Vector3 v, float length, int ticks)
        {
            Laser a = new Laser();
            a.Pos = p;
            a.Scale = length;
            a.Orientation.X = -v.Y;
            a.Orientation.Y = v.X;
            a.Orientation.Z = 0;
            a.Orientation.W = v.Z + v.Length();
            a.Orientation.Normalize();
            a.Color.X = 0.5f;
            a.Color.Y = 1;
            a.Color.Z = 0.25f;
            a.Color.W = 0.9f;
            a.Ticks = ticks;
            a.Vel = v;
            Add(ref a);
        }

        public void AddPlayer(Vector3 p, Vector3 v, float length, int ticks)
        {
            Laser a = new Laser();
            a.Pos = p;
            a.Scale = length;
            a.Orientation.X = -v.Y;
            a.Orientation.Y = v.X;
            a.Orientation.Z = 0;
            a.Orientation.W = v.Z + v.Length();
            a.Orientation.Normalize();
            a.Color.X = 0.6f;
            a.Color.Y = 0.3f;
            a.Color.Z = 0.9f;
            a.Color.W = 0.7f;
            a.Ticks = ticks;
            a.Vel = v;
            Add(ref a);
        }

        public override bool Update(ref Laser a)
        {
            a.Pos += a.Vel;
            a.Vel *= 0.8f;
            a.Color.W *= 0.95f;
            return (--a.Ticks > 0);
        }

        public override void Draw(ref Laser a)
        {
            Shape.AddInstance(a.Pos, a.Scale, a.Orientation, a.Color);
        }

        public void DrawEdge()
        {
            Draw(edgeShape);
        }
    }

    [StructLayout(LayoutKind.Sequential)]
    public struct Laser
    {
        public Vector3 Pos;
        public float Scale;
        public Quaternion Orientation;
        public Vector4 Color;
        public int Ticks;
        public Vector3 Vel;
    }
}
