/*
 * $Id: Plate.cs,v 1.1.1.1 2009/08/10 14:55:11 kenta Exp $
 *
 * Copyright 2009 Kenta Cho. Some rights reserved.
 */
using System;
using System.Runtime.InteropServices;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Abagames.Util;

namespace Abagames.GearToyGear
{
    /// <summary>
    /// Flying score plates.
    /// </summary>
    public class PlatePool : ActorPool<Plate>
    {
        private Frame frame;
        private Field field;
        private int count;

        public PlatePool(int n, Frame frame, Field field)
            : base(n)
        {
            this.frame = frame;
            this.field = field;
        }

        public override void Clear()
        {
            count = 0;
            base.Clear();
        }

        public void Add(float radius, float angle, float z, int number, float scale)
        {
            Plate a = new Plate();
            a.Radius = radius;
            a.TargetRadius = Tube.Radius * (0.9f + (count * 13 % 7) * 0.02f);
            a.Angle = angle;
            a.Z = z;
            a.TargetZ = -160 + (count * 7 % 13) * 3;
            a.Ticks = 120;
            a.Number = 0;
            a.TargetNumber = number;
            a.Alpha = 0;
            a.Scale = scale;
            Add(ref a);
            count++;
        }

        public override bool Update(ref Plate a)
        {
            a.Z += (a.TargetZ - a.Z) * 0.05f;
            a.Radius += (a.TargetRadius - a.Radius) * 0.05f;
            a.Angle += 0.01f;
            if (a.Ticks >= 100)
            {
                a.Alpha += (1 - a.Alpha) * 0.1f;
                a.Number += (int)((a.TargetNumber - a.Number) * 0.1f);
                if (a.Ticks == 100)
                    a.Number = a.TargetNumber;
            }
            if (a.Ticks <= 20)
            {
                a.Alpha += (0 - a.Alpha) * 0.1f;
                a.Angle += 0.05f + 0.1f / a.Ticks;
            }
            return (--a.Ticks > 0);
        }

        public override void Draw()
        {
            ForEach(delegate(ref Plate a)
            {
                Draw(ref a);
            });
        }

        public override void Draw(ref Plate a)
        {
            int n = a.Number;
            float ang = a.Angle;
            Vector3 p = new Vector3();
            p.Z = a.Z;
            for (;;)
            {
                p.X = (float)Math.Sin(ang) * a.Radius;
                p.Y = (float)Math.Cos(ang) * a.Radius;
                Letter.Add(n % 10, p, a.Scale, Quaternion.CreateFromYawPitchRoll(0, 0, (float)Math.PI - ang), a.Alpha);
                n /= 10;
                if (n <= 0)
                    break;
                ang += 0.02f * a.Scale;
            }
        }
    }

    public struct Plate
    {
        public float Radius, TargetRadius;
        public float Angle;
        public float Z, TargetZ;
        public int Ticks;
        public int Number, TargetNumber;
        public float Alpha;
        public float Scale;
    }
}
