/*
 * $Id: PlayerHomingLaser.cs,v 1.1.1.1 2009/08/10 14:55:11 kenta Exp $
 *
 * Copyright 2009 Kenta Cho. Some rights reserved.
 */
using System;
using System.Runtime.InteropServices;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Abagames.Util;

namespace Abagames.GearToyGear
{
    /// <summary>
    /// Players's homing lasers.
    /// </summary>
    public class PlayerHomingLaserPool : ActorPool<PlayerHomingLaser>
    {
        private Frame frame;
        private Field field;
        private LaserPool lasers;
        private Player player;
        private EnemyPool enemies;
        private MiddleEnemyPool middleEnemies;
        private ParticlePool particles;

        public PlayerHomingLaserPool(int n, Frame frame, Field field, LaserPool lasers, Player player, EnemyPool enemies, MiddleEnemyPool middleEnemies, ParticlePool particles)
            : base(n)
        {
            this.frame = frame;
            this.field = field;
            this.lasers = lasers;
            this.player = player;
            this.enemies = enemies;
            this.middleEnemies = middleEnemies;
            this.particles = particles;
        }

        public void Add(bool isMiddleEnemy, int index, float speed, float angle)
        {
            PlayerHomingLaser a = new PlayerHomingLaser();
            a.Pos = player.Pos;
            a.Vel.X = (float)Math.Sin(angle) * speed / 2;
            a.Vel.Y = (float)Math.Cos(angle) * speed / 2;
            a.Vel.Z = 0;
            a.IsAimingMiddleEnemy = isMiddleEnemy;
            a.EnemyIndex = index;
            a.Speed = speed;
            Add(ref a);
        }

        public override void Update()
        {
            int ri = 0;
            int ac = actorCount;
            for (int i = 0; i < actorCount; i++, ri++)
            {
                if (Actors[i].IsRemoved)
                {
                    Remove(i);
                    i--;
                }
            }
            for (int i = 0; i < actorCount; i++)
            {
                if (!Actors[i].IsRemoved)
                    Update(ref Actors[i], i);
            }
        }

        public void Update(ref PlayerHomingLaser a, int index)
        {
            a.Pos += a.Vel;
            a.Vel.Z += (-a.Speed - a.Vel.Z) * 0.05f;
            Vector3 ep;
            if (a.IsAimingMiddleEnemy)
                ep = middleEnemies[a.EnemyIndex].Pos;
            else
                ep = enemies[a.EnemyIndex].Pos;
            float tt = (ep.Z - a.Pos.Z) / a.Vel.Z;
            if (tt < 1)
                tt = 1;
            else if (tt > 100)
                tt = 100;
            float tx = a.Pos.X + a.Vel.X * tt;
            float ty = a.Pos.Y + a.Vel.Y * tt;
            a.Vel.X += (ep.X - a.Pos.X) * (0.2f / tt);
            a.Vel.Y += (ep.Y - a.Pos.Y) * (0.2f / tt);
            a.Pos.X += (ep.X - a.Pos.X) * (1.0f / tt);
            a.Pos.Y += (ep.Y - a.Pos.Y) * (1.0f / tt);
            lasers.AddPlayer(a.Pos, a.Vel, a.Vel.Length() * 0.8f, 40);
            if (a.Pos.Z <= ep.Z)
            {
                if (a.IsAimingMiddleEnemy)
                    middleEnemies.Hit(a.EnemyIndex);
                else
                    enemies.Hit(a.EnemyIndex);
                particles.Add(25, a.Pos, a.Vel, 0.3f, 15, 1, 0.2f, 0.4f, 0.7f);
            }
        }

        public void OnEnemyRemoved(bool isMiddleEnemy, int index)
        {
            for (int i = 0; i < actorCount; i++)
            {
                if (Actors[i].IsAimingMiddleEnemy == isMiddleEnemy)
                {
                    if (Actors[i].EnemyIndex == index)
                        Actors[i].IsRemoved = true;
                    else if (Actors[i].EnemyIndex > index)
                        Actors[i].EnemyIndex--;
                }
            }
        }
    }

    public struct PlayerHomingLaser
    {
        public Vector3 Pos;
        public Vector4 Color;
        public Vector3 Vel;
        public bool IsAimingMiddleEnemy;
        public int EnemyIndex;
        public float Speed;
        public bool IsRemoved;
    }
}
