/*
 * $Id: Shape.cs,v 1.1.1.1 2009/08/10 14:55:11 kenta Exp $
 *
 * Copyright 2009 Kenta Cho. Some rights reserved.
 */
using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using System.Runtime.InteropServices;
using Abagames.Util;

namespace Abagames.GearToyGear
{
    /// <summary>
    /// 3D gear shapes.
    /// </summary>
    public class GearShape : PrimitiveListShape
    {
        private static ushort[] quadIndices = {0, 1, 2, 1, 2, 9, 7, 2, 9, 0, 1, 3, 1, 3, 4, 1, 9, 4, 9, 4, 12, 2, 7, 5, 5, 7, 10, 2, 9, 5, 5, 9, 12};

        public GearShape(Frame frame, float radius, float innerRadius, float thickness, Color frontColor, Color backColor, bool hasCenter)
            : base(frame, 56)
        {
            float a = 0;
            for (int i = 0; i < 8; i++)
            {
                Add((float)Math.Sin(a - 0.1f) * radius, (float)Math.Cos(a - 0.1f) * radius, thickness,
                    frontColor.R, frontColor.G, frontColor.B, frontColor.A);
                Add((float)Math.Sin(a + 0.1f) * radius, (float)Math.Cos(a + 0.1f) * radius, thickness,
                    frontColor.R, frontColor.G, frontColor.B, frontColor.A);
                Add((float)Math.Sin(a) * innerRadius, (float)Math.Cos(a) * innerRadius, thickness,
                    frontColor.R, frontColor.G, frontColor.B, frontColor.A);
                Add((float)Math.Sin(a - 0.1f) * radius, (float)Math.Cos(a - 0.1f) * radius, -thickness,
                    backColor.R, backColor.G, backColor.B, backColor.A);
                Add((float)Math.Sin(a + 0.1f) * radius, (float)Math.Cos(a + 0.1f) * radius, -thickness,
                    backColor.R, backColor.G, backColor.B, backColor.A);
                Add((float)Math.Sin(a) * innerRadius, (float)Math.Cos(a) * innerRadius, -thickness,
                    backColor.R, backColor.G, backColor.B, backColor.A);
                Add((float)Math.Sin(a) * innerRadius, (float)Math.Cos(a) * innerRadius, 0, 0, 0, 0, 0);
                a += (float)Math.PI / 4;
            }
            EndAdd();
            int indexCount;
            if (hasCenter)
                indexCount = 282;
            else
                indexCount = 264;
            indices = new ushort[indexCount * MaxInstanceCount];
            for (int i = 0; i < MaxInstanceCount; i++)
            {
                for (int j = 0; j < 8; j++)
                {
                    for (int k = 0; k < 33; k++)
                    {
                        int idx = quadIndices[k] + j * 7;
                        if (idx >= 56)
                            idx -= 56;
                        indices[i * indexCount + j * 33 + k] = (ushort)(i * 56 + idx);
                    }
                }
                if (hasCenter)
                {
                    for (int j = 0; j < 6; j++)
                    {
                        indices[i * indexCount + 264 + j * 3] = (ushort)(i * 56 + 6);
                        indices[i * indexCount + 264 + j * 3 + 1] = (ushort)(i * 56 + 7 + j * 7);
                        indices[i * indexCount + 264 + j * 3 + 2] = (ushort)(i * 56 + 14 + j * 7);
                    }
                }
            }
            primitiveCount = indexCount / 3;
            CreateIndexBuffer();
        }
    }

    public class GearEdgeShape : PrimitiveListShape
    {
        private static ushort[] quadIndices = {0, 1, 3, 0, 2, 3};

        public GearEdgeShape(Frame frame, float radius, float innerRadius, float thickness, float edgeThickness)
            : base(frame, 48, 48)
        {
            float a = 0;
            float ir = (radius + innerRadius) * 0.5f;
            for (int i = 0; i < 8; i++)
            {
                Add((float)Math.Sin(a - 0.1f) * radius, (float)Math.Cos(a - 0.1f) * radius,
                    thickness, 250, 250, 250, 240);
                Add((float)Math.Sin(a - 0.1f) * (radius - edgeThickness),
                    (float)Math.Cos(a - 0.1f) * (radius - edgeThickness),
                    thickness, 250, 250, 250, 180);
                Add((float)Math.Sin(a + 0.1f) * radius, (float)Math.Cos(a + 0.1f) * radius,
                    thickness, 250, 250, 250, 240);
                Add((float)Math.Sin(a + 0.1f) * (radius - edgeThickness),
                    (float)Math.Cos(a + 0.1f) * (radius - edgeThickness),
                    thickness, 250, 250, 250, 180);
                Add((float)Math.Sin(a + (float)Math.PI / 8) * ir, (float)Math.Cos(a + (float)Math.PI / 8) * ir,
                    thickness, 250, 250, 250, 240);
                Add((float)Math.Sin(a + (float)Math.PI / 8) * (ir - edgeThickness),
                    (float)Math.Cos(a + (float)Math.PI / 8) * (ir - edgeThickness),
                    thickness, 250, 250, 250, 240);
                a += (float)Math.PI / 4;
            }
            EndAdd();
            int indexCount = 144;
            indices = new ushort[indexCount * MaxInstanceCount];
            for (int i = 0; i < MaxInstanceCount; i++)
            {
                for (int j = 0; j < 24; j++)
                {
                    for (int k = 0; k < 6; k++)
                    {
                        int idx = j * 2 + quadIndices[k];
                        if (idx >= 48)
                            idx -= 48;
                        indices[i * indexCount + j * 6 + k] = (ushort)(i * 48 + idx);
                    }
                }
            }
            CreateIndexBuffer();
        }
    }

    public class EllipseShape : PrimitiveListShape
    {
        public EllipseShape(Frame frame, float radius, float ratio)
            : base(frame, 16, 14)
        {
            float a = 0;
            for (int i = 0; i < 16; i++)
            {
                Add((float)Math.Sin(a) * radius * ratio, (float)Math.Cos(a) * radius, 0, 0, 0, 0, 0);
                a += (float)Math.PI / 8;
            }
            EndAdd();
            int indexCount = 42;
            indices = new ushort[indexCount * MaxInstanceCount];
            for (int i = 0; i < MaxInstanceCount; i++)
            {
                for (int j = 0; j < 14; j++)
                {
                    indices[i * indexCount + j * 3] = (ushort)(i * 16);
                    indices[i * indexCount + j * 3 + 1] = (ushort)(i * 16 + 1 + j);
                    indices[i * indexCount + j * 3 + 2] = (ushort)(i * 16 + 2 + j);
                }
            }
            CreateIndexBuffer();
        }
    }

    public class EllipseEdgeShape : PrimitiveListShape
    {
        private static ushort[] quadIndices = {0, 1, 3, 0, 2, 3};

        public EllipseEdgeShape(Frame frame, float radius, float ratio, float innerRadius, int vc)
            : base(frame, vc * 2, vc * 2)
        {
            float a = 0;
            for (int i = 0; i < vc; i++)
            {
                Add((float)Math.Sin(a) * radius * ratio, (float)Math.Cos(a) * radius, 0, 250, 250, 250, 240);
                Add((float)Math.Sin(a) * innerRadius * ratio, (float)Math.Cos(a) * innerRadius, 0, 250, 250, 250, 180);
                a += (float)Math.PI * 2 / vc;
            }
            EndAdd();
            int indexCount = vc * 6;
            indices = new ushort[indexCount * MaxInstanceCount];
            for (int i = 0; i < MaxInstanceCount; i++)
            {
                for (int j = 0; j < vc; j++)
                {
                    for (int k = 0; k < 6; k++)
                    {
                        int idx = j * 2 + quadIndices[k];
                        if (idx >= vc * 2)
                            idx -= vc * 2;
                        indices[i * indexCount + j * 6 + k] = (ushort)(i * vc * 2 + idx);
                    }
                }
            }
            CreateIndexBuffer();
        }
    }

    public class CubeEdgeShape : PrimitiveListShape
    {
        private static ushort[] quadIndices = { 0, 1, 3, 1, 2, 3 };
        private static ushort[] xyEdgeIndices = { 1, 2, 5, 3 };
        private static ushort[] zEdgeIndices = { 0, 1, 13, 12 };
        private static float[,] cubeOffsets = { { -1, -1, -1 }, { 1, -1, -1 }, { 1, 1, -1 }, { -1, 1, -1 }, { -1, -1, 1 }, { 1, -1, 1 }, { 1, 1, 1 }, { -1, 1, 1 } };

        public CubeEdgeShape(Frame frame, float x, float y, float z)
            : base(frame, 24, 24)
        {
            for (int i = 0; i < 8; i++)
            {
                Add(cubeOffsets[i, 0] * x * 1.1f, cubeOffsets[i, 1] * y * 0.9f, cubeOffsets[i, 2] * z * 0.9f, 250, 250, 250, 240);
                Add(cubeOffsets[i, 0] * x * 0.9f, cubeOffsets[i, 1] * y * 1.1f, cubeOffsets[i, 2] * z * 0.9f, 250, 250, 250, 240);
                Add(cubeOffsets[i, 0] * x * 0.9f, cubeOffsets[i, 1] * y * 0.9f, cubeOffsets[i, 2] * z * 1.1f, 250, 250, 250, 240);
            }
            EndAdd();
            indices = new ushort[24 * 3 * MaxInstanceCount];
            ushort[] edgeIndices = new ushort[4];
            for (int i = 0; i < MaxInstanceCount; i++)
            {
                for (int j = 0; j < 12; j++)
                {
                    if (j < 8)
                    {
                        for (int k = 0; k < 4; k++)
                        {
                            ushort idx = (ushort)(xyEdgeIndices[k] + (j % 4) * 3);
                            if (idx >= 12)
                                idx -= 12;
                            if (j >= 4)
                                idx += 12;
                            edgeIndices[k] = idx;
                        }
                    }
                    else
                    {
                        for (int k = 0; k < 4; k++)
                            edgeIndices[k] = (ushort)(zEdgeIndices[k] + (j - 8) * 3);
                    }
                    for (int k = 0; k < 6; k++)
                        indices[i * 24 * 3 + j * 6 + k] = (ushort)(i * 24 + edgeIndices[quadIndices[k]]);
                }
            }
            CreateIndexBuffer();
        }
    }
}
