/*
 * $Id: Shot.cs,v 1.1.1.1 2009/08/10 14:55:11 kenta Exp $
 *
 * Copyright 2009 Kenta Cho. Some rights reserved.
 */
using System;
using System.Runtime.InteropServices;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Abagames.Util;

namespace Abagames.GearToyGear
{
    /// <summary>
    /// Player's shots.
    /// </summary>
    public class ShotPool : ActorPool<Shot>
    {
        private Frame frame;
        private Field field;
        private EnemyPool enemies;
        private MiddleEnemyPool middleEnemies;
        private PillarPool pillars;
        private Player player;
        private ParticlePool particles;
        private Quaternion rollQuaternion;

        public ShotPool(int n, Frame frame, Field field, EnemyPool enemies, MiddleEnemyPool middleEnemies, PillarPool pillars, Player player, ParticlePool particles)
            : base(n)
        {
            this.frame = frame;
            this.field = field;
            this.enemies = enemies;
            this.middleEnemies = middleEnemies;
            this.pillars = pillars;
            this.player = player;
            this.particles = particles;
            shape = new CubeShape(frame, 1, 1, 3);
            rollQuaternion = Quaternion.CreateFromYawPitchRoll(0, 0, 0.05f);
        }

        public void Add(Quaternion o)
        {
            Shot a = new Shot();
            a.Pos = player.Pos;
            Vector3 v = new Vector3(0, 0, -10);
            a.Vel = Vector3.Transform(v, o);
            a.Scale = 2;
            a.Orientation = o;
            a.Color.X = 0.6f;
            a.Color.Y = 0.3f;
            a.Color.Z = 0.9f;
            a.Color.W = 0.9f;
            Add(ref a);
        }

        public override bool Update(ref Shot a)
        {
            a.Orientation *= rollQuaternion;
            a.Pos += a.Vel;
            if (a.Pos.X * a.Pos.X + a.Pos.Y * a.Pos.Y > Tube.Radius * Tube.Radius)
            {
                particles.Add(4, a.Pos, a.Vel, 0.75f, 10, 1.0f, 0.5f, 1.0f, 0.8f);
                return false;
            }
            if (enemies.CheckHit(a.Pos) || middleEnemies.CheckHit(a.Pos) || pillars.CheckHit(a.Pos))
            {
                Vector3 pv = a.Vel;
                pv *= -1;
                particles.Add(4, a.Pos, pv, 0.75f, 10, 1.0f, 0.5f, 1.0f, 0.8f);
                return false;
            }
            return (a.Pos.Z > Field.BackDepth);
        }

        public override void Draw(ref Shot a)
        {
            Shape.AddInstance(a.Pos, a.Scale, a.Orientation, a.Color);
        }
    }

    [StructLayout(LayoutKind.Sequential)]
    public struct Shot
    {
        public Vector3 Pos;
        public float Scale;
        public Quaternion Orientation;
        public Vector4 Color;
        public Vector3 Vel;
    }
}
