/*
 * $Id: Title.cs,v 1.1.1.1 2009/08/10 14:55:11 kenta Exp $
 *
 * Copyright 2009 Kenta Cho. Some rights reserved.
 */
using System;
using System.Runtime.InteropServices;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Abagames.Util;

namespace Abagames.GearToyGear
{
    /// <summary>
    /// Title screen.
    /// </summary>
    public class Title
    {
        private Frame frame;
        private Pad pad;
        private Record record;
        private bool isButtonPressed;
        private int ticks;
        private GearShape shape;
        private GearData[] data;

        public Title(Frame frame, Pad pad, Record record)
        {
            this.frame = frame;
            this.pad = pad;
            this.record = record;
            shape = new GearShape(frame, 1, 0.8f, 0.2f, new Color(255, 255, 255, 255), new Color(255, 255, 255, 255), false);
            data = new GearData[1];
            data[0].Pos.X = -12;
            data[0].Pos.Y = 11;
            data[0].Pos.Z = 0;
            data[0].Color.X = data[0].Color.Y = data[0].Color.Z = data[0].Color.W = 1;
            data[0].Scale = 2;
            data[0].Orientation = Quaternion.Identity;
        }

        public void Initialize()
        {
            isButtonPressed = true;
            ticks = 0;
        }

        public void Update()
        {
            int si = pad.CheckGameStartPressed();
            if (si >= 0)
            {
                if (!isButtonPressed)
                {
                    if (si < 4)
                        pad.SetGamePadPlayerIndex(si);
                    frame.StartGame();
                }
            }
            else
            {
                isButtonPressed = false;
            }
            ticks++;
        }

        public void Draw()
        {
            frame.LookAt(new Vector3(0, 0, 25), Vector3.Zero, new Vector3(0, 1, 0));
            Letter.Add("GEAR", new Vector3(-20, 14, 0), 0.8f, Quaternion.Identity, 1);
            Letter.Add("TOY", new Vector3(-20, 12, 0), 0.6f, Quaternion.Identity, 1);
            Letter.Add("GEAR", new Vector3(-20, 10, 0), 0.8f, Quaternion.Identity, 1);
            Shape.BeginAddInstance();
#if XBOX360
            Shape.SetToWritableVertexBuffer<GearData>(data, 0, 1);
#else
            for (int i = 0; i < 1; i++)
            {
                Shape.AddInstance(data[i].Pos, data[i].Scale, data[i].Orientation, data[i].Color);
            }
#endif
            shape.Draw();
            if (ticks % 60 < 30)
                Letter.Add("PRESS BUTTON", new Vector3(-20, -14, 0), 0.7f, Quaternion.Identity, 1);
            int rc = ticks / 30;
            if (rc > 10)
                rc = 10;
            float y = 14;
            for (int i = 0; i < rc; i++)
            {
                float ls = 0.5f;
                if (record.LastRank == i)
                    ls = 1.0f;
                Letter.Add(i + 1, new Vector3(10, y, 0), ls, Quaternion.Identity, 1);
                Letter.Add(record.Scores[i], new Vector3(20, y, 0), 0.5f, Quaternion.Identity, 1);
                y -= 2;
            }
        }
    }

    [StructLayout(LayoutKind.Sequential)]
    public struct GearData
    {
        public Vector3 Pos;
        public float Scale;
        public Quaternion Orientation;
        public Vector4 Color;
    }
}
