/*
 * $Id: MainLoop.cs,v 1.1.1.1 2009/08/10 14:55:11 kenta Exp $
 *
 * Copyright 2009 Kenta Cho. Some rights reserved.
 */
using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.Content;

namespace Abagames.Util
{
    /// <summary>
    /// A main loop for updating and drawing.
    /// </summary>
    public partial class MainLoop : Microsoft.Xna.Framework.Game
    {
        private Frame frame;
        private GraphicsDeviceManager graphics;

        public MainLoop(Frame frame)
        {
            graphics = new GraphicsDeviceManager(this);
            graphics.PreparingDeviceSettings += new EventHandler<PreparingDeviceSettingsEventArgs>(graphics_PreparingDeviceSettings);
            Content.RootDirectory = "Content";
            this.frame = frame;
            frame.SetParams(graphics, Content, Window, this);
        }

        private void graphics_PreparingDeviceSettings(object sender, PreparingDeviceSettingsEventArgs e)
        {
            PresentationParameters pp = e.GraphicsDeviceInformation.PresentationParameters;
            GraphicsAdapter adapter = e.GraphicsDeviceInformation.Adapter;
            SurfaceFormat format = adapter.CurrentDisplayMode.Format;
            MultiSampleType[] multiSampleTypes = { MultiSampleType.FourSamples, MultiSampleType.TwoSamples };
            foreach (MultiSampleType mst in multiSampleTypes)
            {
                int quality = 0;
                if (adapter.CheckDeviceMultiSampleType(DeviceType.Hardware, format, false, mst, out quality))
                {
                    pp.MultiSampleQuality = 0;
                    pp.MultiSampleType = mst;
                    break;
                }
            }
        }

        protected override void Initialize()
        {
            frame.Initialize();
            base.Initialize();
        }

        protected override void LoadContent()
        {
            frame.LoadContent();
        }

        protected override void UnloadContent()
        {
            frame.UnloadContent();
        }
        
        protected override void Update(GameTime gameTime)
        {
            frame.Update();
            base.Update(gameTime);
        }

        protected override void Draw(GameTime gameTime)
        {
            frame.BeginEffect();
            frame.Draw();
            frame.EndEffect();
            base.Draw(gameTime);
        }
    }
}
