/*
 * $Id: Replay.cs,v 1.1.1.1 2009/08/10 14:55:11 kenta Exp $
 *
 * Copyright 2009 Kenta Cho. Some rights reserved.
 */
using System;
using System.Collections;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Abagames.Util
{
    /// <summary>
    /// Record and replay a user input.
    /// </summary>
    public class Replay
    {
        public int RandomSeed;
        private List<ReplayData> data;
        private List<ReplayEvent> events;
        private ReplayEvent replayEvent;
        private int count;
        private int eventIndex;

        public Replay()
        {
            data = new List<ReplayData>();
            events = new List<ReplayEvent>();
        }

        public void StartRecord()
        {
            data.Clear();
            events.Clear();
            count = 0;
        }

        public void Add(ref ReplayData d)
        {
            data.Add(d);
            count++;
        }

        public void AddEvent()
        {
            replayEvent.Cnt = count;
            events.Add(replayEvent);
        }

        public void StartReplay()
        {
            count = 0;
            eventIndex = 0;
        }

        public bool HasNext()
        {
            return (count < data.Count);
        }

        public ReplayData Get()
        {
            return data[count++];
        }

        public void CheckEvent()
        {
            if (eventIndex >= events.Count ||  events[eventIndex].Cnt != count)
                throw new InvalidReplayException();
            eventIndex++;
        }

        public bool IsAvailable
        {
            get
            {
                return (count > 0);
            }
        }
    }

    public struct ReplayData
    {
        public Vector2 Stick;
        public float LeftTrigger, RightTrigger;
    }

    public struct ReplayEvent
    {
        public int Cnt;
    }

    public class InvalidReplayException : Exception
    {
    }
}
